# S9418.tcl---Silena S9418 ADC module control frame



set frame "S9418"
set frametitle "Silena S9418 ADC Setup & Monitor"
set icontitle "S9418"

set modulename s9418
set moduletype  "Silena S9418"

set defaultmodulelist [list 1 2 3 4 5 6 7 8]

set modulelist {???}
set moduleindex 0

set maxmodule 8
set maxadc 32

if {[info exists env(FAIRbusPresent)] == 0} {
    set maxmodule [inform vmecontrol inquire [set modulename]count]
} else {
    set maxmodule [inform fsysc inquire NumADCs]
}

set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set adclist {}
for {set i 0} {$i < $maxadc} {incr i} {lappend adclist $i}

set module [lindex $modulelist $moduleindex]
set adc  0

set alladcs 0
set allmodules 0


for {set i 0} {$i < $maxadc} {incr i} {
   set adc2r(THRESHOLD$i) "Threshold\[?,??\]"
   set w2r(THRESHOLD$i) "Threshold$i"
}
set w2r(DISABLE) "Disable"
set w2r(RTP)    "RTP"
set w2r(VSN1)   "Vsn1"
set w2r(VSN2)   "Vsn2"
set w2r(TESTAMP) "Test_Reg"
set w2r(TESTMODE) "TestMode"
set w2r(FAIR)    "FAIRMode"
set w2r(BERR)    "Berr_Disable"
set w2r(ENDWD)   "Sep_Enable"
set w2r(SS)      "SS_Disable"
set w2r(TESTPULSE) "TestPulse"

set electronicsvisible 0
set electronicsframe "[self frame-name].electronics"


loadsource common.vme

if {[info exists env(FAIRbusPresent)]} {

proc read-vme-register {r} {
    
    set r1 [split $r #]
    set r2 [split [second $r1] .]

    set modulename [first $r1]
    set module [first $r2]
    set register [second $r2]

    inform fsysc set NetID $module

    switch -glob $register {

        Disable {set data [expr [inform fsysc FAIR_read8 0x422] & 1]}

        RTP  {set data [expr [inform fsysc FAIR_read8 0x427] & 31]}

        Vsn1 {set data [inform fsysc FAIR_read8 0x425]}
        Vsn2 {set data [inform fsysc FAIR_read8 0x424]}

        Threshold* {
            set ch [string range $register 9 end]
            set data [inform fsysc FAIR_read8 [expr 0x400 + $ch]]
        }

        default {
            midas-warning "Unknown register $r"
            set data 0
        }
    }
    return $data
}

proc write-vme-register {r vl} {
    
    set r1 [split $r #]
    set r2 [split [second $r1] .]

    set modulename [first $r1]
    set module [first $r2]
    set register [second $r2]

    inform fsysc set NetID $module

    switch -glob $register {

        Disable {inform fsysc FAIR_write8 0x422 $vl}

        RTP  {inform fsysc FAIR_write8 0x427 $vl}

        Vsn1 {inform fsysc FAIR_write8 0x425 $vl}
        Vsn2 {inform fsysc FAIR_write8 0x424 $vl}

        Threshold* {
            set ch [string range $register 9 end]
            inform fsysc FAIR_write8 [expr 0x400 + $ch] $vl
        }

        default {
            midas-warning "Unknown register $r"
            set data 0
        }
    }
}

proc write-vme-registers {r vl} {
    global allmodules alladcs
    global modulelist module adclist adc

    set module_Current $module

    set r1 [split $r #]
    set r2 [split [second $r1] .]

    set modulename [first $r1]
    set module [first $r2]
    set register [second $r2]

    set-busy

    if {$allmodules} then {set modules $modulelist} else {set modules $module}

    if {[string range $register 0 8] == "Threshold"} then {
        set register "Threshold"
        if {$alladcs} then {set adcs $adclist} else {set adcs $adc}
        foreach i $modules {
           if {$allmodules && $alladcs} then {set-footer "setting module $i"}
           set module $i
           foreach j $adcs {
               write-vme-register $modulename#$i.$register$j $vl
           }
        }
    } else {
        foreach i $modules {
           set module $i
           write-vme-register $modulename#$i.$register $vl
        }
    }
    set module $module_Current

    clear-busy
}

}


proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :transient T

    :galley G :deaf T :strings $text :font fixed :rows 50
    :show T
}


proc click-widget {w value} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

#    click-redisplay
}

proc click-adc-widget {w value} {
    global w2r adc2r
    global modulename module
    global alladcs allmodules

    if {$alladcs == 0} then {set register $w2r($w)} else {set register $adc2r($w)}

    if {$alladcs == 0} then {
        if {$allmodules == 0} then {
            write-vme-register $modulename#$module.$register $value
        } else {
            write-vme-registers $modulename#\[?,??\].$register $value
        }
    } else {
        if {$allmodules == 0} then {
            write-vme-registers $modulename#$module.$register $value
        } else {
            write-vme-registers $modulename#\[?,??\].$register $value
        }
    }

    click-redisplay
}

proc reset-module {} {
    global modulename module allmodules
}

proc generate_test_pulse {} {click-widget TESTPULSE 255}
    

proc do_read_fifo {} {
  global CurrentVMEServer modulename module

  set info ""
  for {set Offset 0} {$Offset < 1088} {incr Offset 64} {

     set register $modulename#$module.FIFO
     set data [string trimleft [read-vme-register $register]]

     append info "\nData Register (FIFO) Offset $Offset\n\n"
     foreach word $data {
         if {$word == 0x7fffffff} then {
             append info "$word - assumed end of data\n"
             midas-information $info
             return
         }
         set type [expr $word >> 30 & 3]
         switch $type {
           0 {
             set event_type "data"
             set channel [expr $word >> 20 & 0x1f]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i;  adc= 0x%3x" $word $event_type $channel $adc]"
           }
           1 {
             set event_type "footer"
             set VSN [expr $word & 0xffff]
             set WC [expr $word >> 16 & 0x3f]
             set EvCount [expr $word >> 25 & 0xf]
             append info "[format "0x%8x    %8s   vsn= 0x%4x;   wc= %3i; Ev Counter= %3i" $word $event_type $VSN $WC $EvCount]"
           }
           2 -
           3 {
             set event_type "end event"
             append info "[format "0x%8x    %8s" $word $event_type]"
           }
         }
         append info "\n"
     }
  }
  append info "end of fifo reached\n"
  midas-information $info
}

proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-electronics {} {
#  display electronics sub frame
   global electronicsvisible electronicsframe
   if {$electronicsvisible==0} then {make-electronics} else {open-frame $electronicsframe}
   click-redisplay
}

proc click-select-module {v} {
   global debug module maxmodule
   if {$debug} then {puts stderr "click-select-module with argument \{$v\}"}
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-select-adc {v} {}

proc click-all-adcs {v} {global alladcs; set alladcs $v}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc repaint {} {}

proc redisplay-module {} {
   global env w2r modulename module maxmodule

   if {[info exists env(FAIRbusPresent)] == 0} {
       set maxmodule [inform vmecontrol inquire [set modulename]count]
   } else {
       set maxmodule [inform fsysc inquire NumADCs]
   }

   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module

   foreach w {DISABLE RTP VSN1 VSN2} {
       set register $w2r($w)
       set value [read-vme-register $modulename#$module.$register]
       :modify $w :value $value
   }  
}

proc redisplay-adcs {} {
    global w2r
    global modulename module maxadc

    foreach W {THRESHOLD} {
       for {set i 0} {$i < $maxadc} {incr i} {
          set w $W$i
          set register $w2r($w)
          set value [read-vme-register $modulename#$module.$register]
          :modify $w :value $value
       }
    }
}

proc redisplay-electronics {} {
   global electronicsvisible
   global modulename module w2r

   if {$electronicsvisible==0} then {return}

   foreach w {TESTAMP TESTMODE FAIR BERR ENDWD SS} {
       set register $w2r($w)
       set value [read-vme-register $modulename#$module.$register]
       :modify $w :value $value
   }
}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set EGreport -1
   set-busy

   redisplay-module
   redisplay-adcs
   redisplay-electronics

   clear-busy
   set EGreport 0
}

proc start args {
   global env started modulename maxmodule modulelist moduleindex module

   if {$started == 0} then {
       make-frame
       EGbindings
   }
   set started 1
   open-frame

   clear-footer
   clear-busy

   loadsource [set modulename].defn

   if {[info exists env(FAIRbusPresent)] == 0} {
       set maxmodule [inform vmecontrol inquire [set modulename]count]
   } else {
       set maxmodule [inform fsysc inquire NumADCs]
   }

   set modulelist {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

   set moduleindex 0
   set module [lindex $modulelist $moduleindex]

   :modify MODULE-SEL :strings $modulelist
   click-select-module $module

   click-redisplay
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}


proc click arg {
   global debug actionlist
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      RTP -
      VSN1 -
      VSN2 -
      TESTAMP -
      TESTMODE -
      FAIR -
      BERR -
      ENDWD -
      SS -
      DISABLE       {click-widget $w $v}

      THRESHOLD*  {click-adc-widget $w $v}

      ENGINEERING       {click-$v}
      REDISPLAY  	{click-redisplay}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env actionlist
  global module maxmodule modulelist maxadc

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :hskip 50
     :checkbox ALL-MODULES :label "Act on ALL modules?"
     :next-row T

     :message M1 :label "Module Setup" :bold T
     :next-row T

     :slider RTP       :label "Rise Time Protect"  :min 0 :max 31
     :text VSN1        :label VSN1 :w 4
     :text VSN2        :label VSN2 :w 4
     :hskip 50
     :checkbox DISABLE
  
   :panel ADC :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 0
     :message M2 :label "ADC Thresholds" :bold T
     :hskip 400
     :checkbox ALL-ADCS :label "Act on ALL ADCs?"
     :next-row T

     for {set i 0} {$i < $maxadc} {} {
         :slider   THRESHOLD$i :label "$i" :min 0 :max 255 :entry-width 4 :valuex 2c
         :hskip 10
         incr i
         if {[expr $i / 4 * 4] == $i} {:next-row T}
     }
  
   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software registers] 
      :button MEMRD   :label "read data FIFO" :command do_read_fifo

  :show T

}

proc make-electronics {} {
  global env electronicsframe electronicsvisible

 set valuex 16c

  :frame "$electronicsframe" :label "Silena S9418 ADC Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
     :layout V; :row-gap 5; :col-gap 10

      :slider TESTAMP    :label "Test pulse amplitude"  :min 0 :max 255
      :button TESTGEN    :label "Generate test pulse" :command generate_test_pulse
      :next-row T

      :message EM1 :label "Control Register 1" :bold T
      :choice TESTMODE   :label "TestMode"  :valuex $valuex  :strings [list disabled enabled]
      :choice FAIR       :label "FAIRbus"   :valuex $valuex  :strings [list disabled enabled]
      :next-row T

      :message EM2 :label "Control Register 2" :bold T
      :choice BERR   :label "Bus error"  :valuex $valuex      :strings [list enabled disabled]
      :choice ENDWD  :label "Separators"   :valuex $valuex    :strings [list 1 2]
      :choice SS     :label "Sliding Scale"  :valuex $valuex  :strings [list enabled disabled]

    :next-row T
     :layout H; :col-gap 10
     :button UPDATE   :label "Electronics Update"   :command redisplay-electronics

  :show T
 set electronicsvisible 1
}

